@@ -987,6 +987,8 @@ pub enum CacheBucket {
     Environments,
     /// Cached Python downloads
     Python,
+    /// Downloaded tool binaries (e.g., Ruff).
+    Binaries,
 }
 
 impl CacheBucket {
@@ -1000,7 +1002,7 @@ impl CacheBucket {
             Self::Interpreter => "interpreter-v4",
             // Note that when bumping this, you'll also need to bump it
             // in `crates/uv/tests/it/cache_clean.rs`.
-            Self::Simple => "simple-v16",
+            Self::Simple => "simple-v17",
             // Note that when bumping this, you'll also need to bump it
             // in `crates/uv/tests/it/cache_prune.rs`.
             Self::Wheels => "wheels-v5",
@@ -1010,6 +1012,7 @@ impl CacheBucket {
             Self::Builds => "builds-v0",
             Self::Environments => "environments-v2",
             Self::Python => "python-v0",
+            Self::Binaries => "binaries-v0",
         }
     }
 
@@ -1116,7 +1119,8 @@ impl CacheBucket {
             | Self::Archive
             | Self::Builds
             | Self::Environments
-            | Self::Python => {
+            | Self::Python
+            | Self::Binaries => {
                 // Nothing to do.
             }
         }
@@ -1135,6 +1139,7 @@ impl CacheBucket {
             Self::Archive,
             Self::Builds,
             Self::Environments,
+            Self::Binaries,
         ]
         .iter()
         .copied()
@@ -1212,35 +1217,30 @@ impl Refresh {
     /// Combine two [`Refresh`] policies, taking the "max" of the two policies.
     #[must_use]
     pub fn combine(self, other: Self) -> Self {
-        /// Return the maximum of two timestamps.
-        fn max(a: Timestamp, b: Timestamp) -> Timestamp {
-            if a > b { a } else { b }
-        }
-
         match (self, other) {
             // If the policy is `None`, return the existing refresh policy.
             // Take the `max` of the two timestamps.
-            (Self::None(t1), Self::None(t2)) => Self::None(max(t1, t2)),
-            (Self::None(t1), Self::All(t2)) => Self::All(max(t1, t2)),
+            (Self::None(t1), Self::None(t2)) => Self::None(t1.max(t2)),
+            (Self::None(t1), Self::All(t2)) => Self::All(t1.max(t2)),
             (Self::None(t1), Self::Packages(packages, paths, t2)) => {
-                Self::Packages(packages, paths, max(t1, t2))
+                Self::Packages(packages, paths, t1.max(t2))
             }
 
             // If the policy is `All`, refresh all packages.
-            (Self::All(t1), Self::None(t2)) => Self::All(max(t1, t2)),
-            (Self::All(t1), Self::All(t2)) => Self::All(max(t1, t2)),
-            (Self::All(t1), Self::Packages(.., t2)) => Self::All(max(t1, t2)),
+            (Self::All(t1), Self::None(t2) | Self::All(t2) | Self::Packages(.., t2)) => {
+                Self::All(t1.max(t2))
+            }
 
             // If the policy is `Packages`, take the "max" of the two policies.
             (Self::Packages(packages, paths, t1), Self::None(t2)) => {
-                Self::Packages(packages, paths, max(t1, t2))
+                Self::Packages(packages, paths, t1.max(t2))
             }
-            (Self::Packages(.., t1), Self::All(t2)) => Self::All(max(t1, t2)),
+            (Self::Packages(.., t1), Self::All(t2)) => Self::All(t1.max(t2)),
             (Self::Packages(packages1, paths1, t1), Self::Packages(packages2, paths2, t2)) => {
                 Self::Packages(
                     packages1.into_iter().chain(packages2).collect(),
                     paths1.into_iter().chain(paths2).collect(),
-                    max(t1, t2),
+                    t1.max(t2),
                 )
             }
         }
