@@ -4,15 +4,15 @@ use serde::{Deserialize, Serialize};
 
 use uv_cache_info::CacheKey;
 use uv_configuration::{
-    ConfigSettings, IndexStrategy, KeyringProviderType, PackageConfigSettings,
-    PackageNameSpecifier, RequiredVersion, TargetTriple, TrustedHost, TrustedPublishing,
+    BuildIsolation, IndexStrategy, KeyringProviderType, PackageNameSpecifier, Reinstall,
+    RequiredVersion, TargetTriple, TrustedHost, TrustedPublishing, Upgrade,
 };
 use uv_distribution_types::{
-    Index, IndexUrl, IndexUrlError, PipExtraIndex, PipFindLinks, PipIndex, StaticMetadata,
+    ConfigSettings, ExtraBuildVariables, Index, IndexUrl, IndexUrlError, PackageConfigSettings,
+    PipExtraIndex, PipFindLinks, PipIndex, StaticMetadata,
 };
 use uv_install_wheel::LinkMode;
 use uv_macros::{CombineOptions, OptionsMetadata};
-
 use uv_normalize::{ExtraName, PackageName, PipGroupName};
 use uv_pep508::Requirement;
 use uv_pypi_types::{SupportedEnvironments, VerbatimParsedUrl};
@@ -24,7 +24,8 @@ use uv_resolver::{
 };
 use uv_static::EnvVars;
 use uv_torch::TorchMode;
-use uv_workspace::{pyproject::ExtraBuildDependencies, pyproject_mut::AddBoundsKind};
+use uv_workspace::pyproject::ExtraBuildDependencies;
+use uv_workspace::pyproject_mut::AddBoundsKind;
 
 /// A `pyproject.toml` with an (optional) `[tool.uv]` section.
 #[allow(dead_code)]
@@ -51,7 +52,7 @@ pub struct Options {
     pub globals: GlobalOptions,
 
     #[serde(flatten)]
-    pub top_level: ResolverInstallerOptions,
+    pub top_level: ResolverInstallerSchema,
 
     #[serde(flatten)]
     pub install_mirrors: PythonInstallMirrors,
@@ -160,7 +161,7 @@ pub struct Options {
 
 impl Options {
     /// Construct an [`Options`] with the given global and top-level settings.
-    pub fn simple(globals: GlobalOptions, top_level: ResolverInstallerOptions) -> Self {
+    pub fn simple(globals: GlobalOptions, top_level: ResolverInstallerSchema) -> Self {
         Self {
             globals,
             top_level,
@@ -340,13 +341,12 @@ pub struct InstallerOptions {
     pub exclude_newer: Option<ExcludeNewerTimestamp>,
     pub link_mode: Option<LinkMode>,
     pub compile_bytecode: Option<bool>,
-    pub reinstall: Option<bool>,
-    pub reinstall_package: Option<Vec<PackageName>>,
+    pub reinstall: Option<Reinstall>,
+    pub build_isolation: Option<BuildIsolation>,
     pub no_build: Option<bool>,
     pub no_build_package: Option<Vec<PackageName>>,
     pub no_binary: Option<bool>,
     pub no_binary_package: Option<Vec<PackageName>>,
-    pub no_build_isolation: Option<bool>,
     pub no_sources: Option<bool>,
 }
 
@@ -368,24 +368,171 @@ pub struct ResolverOptions {
     pub config_settings_package: Option<PackageConfigSettings>,
     pub exclude_newer: ExcludeNewer,
     pub link_mode: Option<LinkMode>,
-    pub upgrade: Option<bool>,
-    pub upgrade_package: Option<Vec<Requirement<VerbatimParsedUrl>>>,
+    pub upgrade: Option<Upgrade>,
+    pub build_isolation: Option<BuildIsolation>,
     pub no_build: Option<bool>,
     pub no_build_package: Option<Vec<PackageName>>,
     pub no_binary: Option<bool>,
     pub no_binary_package: Option<Vec<PackageName>>,
-    pub no_build_isolation: Option<bool>,
-    pub no_build_isolation_package: Option<Vec<PackageName>>,
     pub extra_build_dependencies: Option<ExtraBuildDependencies>,
+    pub extra_build_variables: Option<ExtraBuildVariables>,
     pub no_sources: Option<bool>,
 }
 
 /// Shared settings, relevant to all operations that must resolve and install dependencies. The
 /// union of [`InstallerOptions`] and [`ResolverOptions`].
+#[derive(Debug, Clone, Default, CombineOptions)]
+pub struct ResolverInstallerOptions {
+    pub index: Option<Vec<Index>>,
+    pub index_url: Option<PipIndex>,
+    pub extra_index_url: Option<Vec<PipExtraIndex>>,
+    pub no_index: Option<bool>,
+    pub find_links: Option<Vec<PipFindLinks>>,
+    pub index_strategy: Option<IndexStrategy>,
+    pub keyring_provider: Option<KeyringProviderType>,
+    pub resolution: Option<ResolutionMode>,
+    pub prerelease: Option<PrereleaseMode>,
+    pub fork_strategy: Option<ForkStrategy>,
+    pub dependency_metadata: Option<Vec<StaticMetadata>>,
+    pub config_settings: Option<ConfigSettings>,
+    pub config_settings_package: Option<PackageConfigSettings>,
+    pub build_isolation: Option<BuildIsolation>,
+    pub extra_build_dependencies: Option<ExtraBuildDependencies>,
+    pub extra_build_variables: Option<ExtraBuildVariables>,
+    pub exclude_newer: Option<ExcludeNewerTimestamp>,
+    pub exclude_newer_package: Option<ExcludeNewerPackage>,
+    pub link_mode: Option<LinkMode>,
+    pub compile_bytecode: Option<bool>,
+    pub no_sources: Option<bool>,
+    pub upgrade: Option<Upgrade>,
+    pub reinstall: Option<Reinstall>,
+    pub no_build: Option<bool>,
+    pub no_build_package: Option<Vec<PackageName>>,
+    pub no_binary: Option<bool>,
+    pub no_binary_package: Option<Vec<PackageName>>,
+}
+
+impl From<ResolverInstallerSchema> for ResolverInstallerOptions {
+    fn from(value: ResolverInstallerSchema) -> Self {
+        let ResolverInstallerSchema {
+            index,
+            index_url,
+            extra_index_url,
+            no_index,
+            find_links,
+            index_strategy,
+            keyring_provider,
+            resolution,
+            prerelease,
+            fork_strategy,
+            dependency_metadata,
+            config_settings,
+            config_settings_package,
+            no_build_isolation,
+            no_build_isolation_package,
+            extra_build_dependencies,
+            extra_build_variables,
+            exclude_newer,
+            exclude_newer_package,
+            link_mode,
+            compile_bytecode,
+            no_sources,
+            upgrade,
+            upgrade_package,
+            reinstall,
+            reinstall_package,
+            no_build,
+            no_build_package,
+            no_binary,
+            no_binary_package,
+        } = value;
+        Self {
+            index,
+            index_url,
+            extra_index_url,
+            no_index,
+            find_links,
+            index_strategy,
+            keyring_provider,
+            resolution,
+            prerelease,
+            fork_strategy,
+            dependency_metadata,
+            config_settings,
+            config_settings_package,
+            build_isolation: BuildIsolation::from_args(
+                no_build_isolation,
+                no_build_isolation_package.into_iter().flatten().collect(),
+            ),
+            extra_build_dependencies,
+            extra_build_variables,
+            exclude_newer,
+            exclude_newer_package,
+            link_mode,
+            compile_bytecode,
+            no_sources,
+            upgrade: Upgrade::from_args(
+                upgrade,
+                upgrade_package
+                    .into_iter()
+                    .flatten()
+                    .map(Into::into)
+                    .collect(),
+            ),
+            reinstall: Reinstall::from_args(reinstall, reinstall_package.unwrap_or_default()),
+            no_build,
+            no_build_package,
+            no_binary,
+            no_binary_package,
+        }
+    }
+}
+
+impl ResolverInstallerSchema {
+    /// Resolve the [`ResolverInstallerSchema`] relative to the given root directory.
+    pub fn relative_to(self, root_dir: &Path) -> Result<Self, IndexUrlError> {
+        Ok(Self {
+            index: self
+                .index
+                .map(|index| {
+                    index
+                        .into_iter()
+                        .map(|index| index.relative_to(root_dir))
+                        .collect::<Result<Vec<_>, _>>()
+                })
+                .transpose()?,
+            index_url: self
+                .index_url
+                .map(|index_url| index_url.relative_to(root_dir))
+                .transpose()?,
+            extra_index_url: self
+                .extra_index_url
+                .map(|extra_index_url| {
+                    extra_index_url
+                        .into_iter()
+                        .map(|extra_index_url| extra_index_url.relative_to(root_dir))
+                        .collect::<Result<Vec<_>, _>>()
+                })
+                .transpose()?,
+            find_links: self
+                .find_links
+                .map(|find_links| {
+                    find_links
+                        .into_iter()
+                        .map(|find_link| find_link.relative_to(root_dir))
+                        .collect::<Result<Vec<_>, _>>()
+                })
+                .transpose()?,
+            ..self
+        })
+    }
+}
+
+/// The JSON schema for the `[tool.uv]` section of a `pyproject.toml` file.
 #[derive(Debug, Clone, Default, PartialEq, Eq, Deserialize, CombineOptions, OptionsMetadata)]
 #[serde(rename_all = "kebab-case")]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
-pub struct ResolverInstallerOptions {
+pub struct ResolverInstallerSchema {
     /// The package indexes to use when resolving dependencies.
     ///
     /// Accepts either a repository compliant with [PEP 503](https://peps.python.org/pep-0503/)
@@ -638,11 +785,22 @@ pub struct ResolverInstallerOptions {
         default = "[]",
         value_type = "dict",
         example = r#"
-        [extra-build-dependencies] 
-        pytest = ["setuptools"]
-    "#
+            extra-build-dependencies = { pytest = ["setuptools"] }
+        "#
     )]
     pub extra_build_dependencies: Option<ExtraBuildDependencies>,
+    /// Extra environment variables to set when building certain packages.
+    ///
+    /// Environment variables will be added to the environment when building the
+    /// specified packages.
+    #[option(
+        default = r#"{}"#,
+        value_type = r#"dict[str, dict[str, str]]"#,
+        example = r#"
+            extra-build-variables = { flash-attn = { FLASH_ATTENTION_SKIP_CUDA_BUILD = "TRUE" } }
+        "#
+    )]
+    pub extra_build_variables: Option<ExtraBuildVariables>,
     /// Limit candidate packages to those that were uploaded prior to a given point in time.
     ///
     /// Accepts a superset of [RFC 3339](https://www.rfc-editor.org/rfc/rfc3339.html) (e.g.,
@@ -671,6 +829,11 @@ pub struct ResolverInstallerOptions {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
+    ///
+    /// WARNING: The use of symlink link mode is discouraged, as they create tight coupling between
+    /// the cache and the target environment. For example, clearing the cache (`uv cache clean`)
+    /// will break all installed packages by way of removing the underlying source files. Use
+    /// symlinks with caution.
     #[option(
         default = "\"clone\" (macOS) or \"hardlink\" (Linux, Windows)",
         value_type = "str",
@@ -794,46 +957,6 @@ pub struct ResolverInstallerOptions {
     pub no_binary_package: Option<Vec<PackageName>>,
 }
 
-impl ResolverInstallerOptions {
-    /// Resolve the [`ResolverInstallerOptions`] relative to the given root directory.
-    pub fn relative_to(self, root_dir: &Path) -> Result<Self, IndexUrlError> {
-        Ok(Self {
-            index: self
-                .index
-                .map(|index| {
-                    index
-                        .into_iter()
-                        .map(|index| index.relative_to(root_dir))
-                        .collect::<Result<Vec<_>, _>>()
-                })
-                .transpose()?,
-            index_url: self
-                .index_url
-                .map(|index_url| index_url.relative_to(root_dir))
-                .transpose()?,
-            extra_index_url: self
-                .extra_index_url
-                .map(|extra_index_url| {
-                    extra_index_url
-                        .into_iter()
-                        .map(|extra_index_url| extra_index_url.relative_to(root_dir))
-                        .collect::<Result<Vec<_>, _>>()
-                })
-                .transpose()?,
-            find_links: self
-                .find_links
-                .map(|find_links| {
-                    find_links
-                        .into_iter()
-                        .map(|find_link| find_link.relative_to(root_dir))
-                        .collect::<Result<Vec<_>, _>>()
-                })
-                .transpose()?,
-            ..self
-        })
-    }
-}
-
 /// Shared settings, relevant to all operations that might create managed python installations.
 #[derive(Debug, Clone, PartialEq, Eq, Deserialize, CombineOptions, OptionsMetadata)]
 #[serde(rename_all = "kebab-case")]
@@ -886,7 +1009,7 @@ pub struct PythonInstallMirrors {
 
 impl Default for PythonInstallMirrors {
     fn default() -> Self {
-        PythonInstallMirrors::resolve(None, None, None)
+        Self::resolve(None, None, None)
     }
 }
 
@@ -900,7 +1023,7 @@ impl PythonInstallMirrors {
         let pypy_mirror_env = std::env::var(EnvVars::UV_PYPY_INSTALL_MIRROR).ok();
         let python_downloads_json_url_env =
             std::env::var(EnvVars::UV_PYTHON_DOWNLOADS_JSON_URL).ok();
-        PythonInstallMirrors {
+        Self {
             python_install_mirror: python_mirror_env.or(python_mirror),
             pypy_install_mirror: pypy_mirror_env.or(pypy_mirror),
             python_downloads_json_url: python_downloads_json_url_env.or(python_downloads_json_url),
@@ -1159,11 +1282,22 @@ pub struct PipOptions {
         default = "[]",
         value_type = "dict",
         example = r#"
-            [extra-build-dependencies]
-            pytest = ["setuptools"]
+            extra-build-dependencies = { pytest = ["setuptools"] }
         "#
     )]
     pub extra_build_dependencies: Option<ExtraBuildDependencies>,
+    /// Extra environment variables to set when building certain packages.
+    ///
+    /// Environment variables will be added to the environment when building the
+    /// specified packages.
+    #[option(
+        default = r#"{}"#,
+        value_type = r#"dict[str, dict[str, str]]"#,
+        example = r#"
+            extra-build-variables = { flash-attn = { FLASH_ATTENTION_SKIP_CUDA_BUILD = "TRUE" } }
+        "#
+    )]
+    pub extra_build_variables: Option<ExtraBuildVariables>,
     /// Validate the Python environment, to detect packages with missing dependencies and other
     /// issues.
     #[option(
@@ -1541,6 +1675,11 @@ pub struct PipOptions {
     ///
     /// Defaults to `clone` (also known as Copy-on-Write) on macOS, and `hardlink` on Linux and
     /// Windows.
+    ///
+    /// WARNING: The use of symlink link mode is discouraged, as they create tight coupling between
+    /// the cache and the target environment. For example, clearing the cache (`uv cache clean`)
+    /// will break all installed packages by way of removing the underlying source files. Use
+    /// symlinks with caution.
     #[option(
         default = "\"clone\" (macOS) or \"hardlink\" (Linux, Windows)",
         value_type = "str",
@@ -1714,8 +1853,8 @@ impl PipOptions {
     }
 }
 
-impl From<ResolverInstallerOptions> for ResolverOptions {
-    fn from(value: ResolverInstallerOptions) -> Self {
+impl From<ResolverInstallerSchema> for ResolverOptions {
+    fn from(value: ResolverInstallerSchema) -> Self {
         Self {
             index: value.index,
             index_url: value.index_url,
@@ -1740,22 +1879,32 @@ impl From<ResolverInstallerOptions> for ResolverOptions {
                     .collect(),
             ),
             link_mode: value.link_mode,
-            upgrade: value.upgrade,
-            upgrade_package: value.upgrade_package,
+            upgrade: Upgrade::from_args(
+                value.upgrade,
+                value
+                    .upgrade_package
+                    .into_iter()
+                    .flatten()
+                    .map(Into::into)
+                    .collect(),
+            ),
             no_build: value.no_build,
             no_build_package: value.no_build_package,
             no_binary: value.no_binary,
             no_binary_package: value.no_binary_package,
-            no_build_isolation: value.no_build_isolation,
-            no_build_isolation_package: value.no_build_isolation_package,
+            build_isolation: BuildIsolation::from_args(
+                value.no_build_isolation,
+                value.no_build_isolation_package.unwrap_or_default(),
+            ),
             extra_build_dependencies: value.extra_build_dependencies,
+            extra_build_variables: value.extra_build_variables,
             no_sources: value.no_sources,
         }
     }
 }
 
-impl From<ResolverInstallerOptions> for InstallerOptions {
-    fn from(value: ResolverInstallerOptions) -> Self {
+impl From<ResolverInstallerSchema> for InstallerOptions {
+    fn from(value: ResolverInstallerSchema) -> Self {
         Self {
             index: value.index,
             index_url: value.index_url,
@@ -1777,13 +1926,18 @@ impl From<ResolverInstallerOptions> for InstallerOptions {
             .global,
             link_mode: value.link_mode,
             compile_bytecode: value.compile_bytecode,
-            reinstall: value.reinstall,
-            reinstall_package: value.reinstall_package,
+            reinstall: Reinstall::from_args(
+                value.reinstall,
+                value.reinstall_package.unwrap_or_default(),
+            ),
+            build_isolation: BuildIsolation::from_args(
+                value.no_build_isolation,
+                value.no_build_isolation_package.unwrap_or_default(),
+            ),
             no_build: value.no_build,
             no_build_package: value.no_build_package,
             no_binary: value.no_binary,
             no_binary_package: value.no_binary_package,
-            no_build_isolation: value.no_build_isolation,
             no_sources: value.no_sources,
         }
     }
@@ -1791,7 +1945,7 @@ impl From<ResolverInstallerOptions> for InstallerOptions {
 
 /// The options persisted alongside an installed tool.
 ///
-/// A mirror of [`ResolverInstallerOptions`], without upgrades and reinstalls, which shouldn't be
+/// A mirror of [`ResolverInstallerSchema`], without upgrades and reinstalls, which shouldn't be
 /// persisted in a tool receipt.
 #[derive(
     Debug, Clone, Default, PartialEq, Eq, Serialize, Deserialize, CombineOptions, OptionsMetadata,
@@ -1812,9 +1966,9 @@ pub struct ToolOptions {
     pub dependency_metadata: Option<Vec<StaticMetadata>>,
     pub config_settings: Option<ConfigSettings>,
     pub config_settings_package: Option<PackageConfigSettings>,
-    pub no_build_isolation: Option<bool>,
-    pub no_build_isolation_package: Option<Vec<PackageName>>,
+    pub build_isolation: Option<BuildIsolation>,
     pub extra_build_dependencies: Option<ExtraBuildDependencies>,
+    pub extra_build_variables: Option<ExtraBuildVariables>,
     pub exclude_newer: Option<ExcludeNewerTimestamp>,
     pub exclude_newer_package: Option<ExcludeNewerPackage>,
     pub link_mode: Option<LinkMode>,
@@ -1842,9 +1996,9 @@ impl From<ResolverInstallerOptions> for ToolOptions {
             dependency_metadata: value.dependency_metadata,
             config_settings: value.config_settings,
             config_settings_package: value.config_settings_package,
-            no_build_isolation: value.no_build_isolation,
-            no_build_isolation_package: value.no_build_isolation_package,
+            build_isolation: value.build_isolation,
             extra_build_dependencies: value.extra_build_dependencies,
+            extra_build_variables: value.extra_build_variables,
             exclude_newer: value.exclude_newer,
             exclude_newer_package: value.exclude_newer_package,
             link_mode: value.link_mode,
@@ -1874,18 +2028,16 @@ impl From<ToolOptions> for ResolverInstallerOptions {
             dependency_metadata: value.dependency_metadata,
             config_settings: value.config_settings,
             config_settings_package: value.config_settings_package,
-            no_build_isolation: value.no_build_isolation,
-            no_build_isolation_package: value.no_build_isolation_package,
+            build_isolation: value.build_isolation,
             extra_build_dependencies: value.extra_build_dependencies,
+            extra_build_variables: value.extra_build_variables,
             exclude_newer: value.exclude_newer,
             exclude_newer_package: value.exclude_newer_package,
             link_mode: value.link_mode,
             compile_bytecode: value.compile_bytecode,
             no_sources: value.no_sources,
             upgrade: None,
-            upgrade_package: None,
             reinstall: None,
-            reinstall_package: None,
             no_build: value.no_build,
             no_build_package: value.no_build_package,
             no_binary: value.no_binary,
@@ -1932,6 +2084,7 @@ pub struct OptionsWire {
     no_build_isolation: Option<bool>,
     no_build_isolation_package: Option<Vec<PackageName>>,
     extra_build_dependencies: Option<ExtraBuildDependencies>,
+    extra_build_variables: Option<ExtraBuildVariables>,
     exclude_newer: Option<ExcludeNewerTimestamp>,
     exclude_newer_package: Option<ExcludeNewerPackage>,
     link_mode: Option<LinkMode>,
@@ -2052,6 +2205,7 @@ impl From<OptionsWire> for Options {
             default_groups,
             dependency_groups,
             extra_build_dependencies,
+            extra_build_variables,
             dev_dependencies,
             managed,
             package,
@@ -2076,7 +2230,7 @@ impl From<OptionsWire> for Options {
                 // Used twice for backwards compatibility
                 allow_insecure_host: allow_insecure_host.clone(),
             },
-            top_level: ResolverInstallerOptions {
+            top_level: ResolverInstallerSchema {
                 index,
                 index_url,
                 extra_index_url,
@@ -2093,6 +2247,7 @@ impl From<OptionsWire> for Options {
                 no_build_isolation,
                 no_build_isolation_package,
                 extra_build_dependencies,
+                extra_build_variables,
                 exclude_newer,
                 exclude_newer_package,
                 link_mode,
