@@ -45,6 +45,9 @@ impl EnvVars {
     /// directory for caching instead of the default cache directory.
     pub const UV_CACHE_DIR: &'static str = "UV_CACHE_DIR";
 
+    /// The directory for storage of credentials when using a plain text backend.
+    pub const UV_CREDENTIALS_DIR: &'static str = "UV_CREDENTIALS_DIR";
+
     /// Equivalent to the `--no-cache` command-line argument. If set, uv will not use the
     /// cache for any operations.
     pub const UV_NO_CACHE: &'static str = "UV_NO_CACHE";
@@ -136,6 +139,10 @@ impl EnvVars {
     /// directories.
     pub const UV_NO_CONFIG: &'static str = "UV_NO_CONFIG";
 
+    /// Equivalent to the `--isolated` command-line argument. If set, uv will avoid discovering
+    /// a `pyproject.toml` or `uv.toml` file.
+    pub const UV_ISOLATED: &'static str = "UV_ISOLATED";
+
     /// Equivalent to the `--exclude-newer` command-line argument. If set, uv will
     /// exclude distributions published after the specified date.
     pub const UV_EXCLUDE_NEWER: &'static str = "UV_EXCLUDE_NEWER";
@@ -170,6 +177,14 @@ impl EnvVars {
     /// members, as non-editable.
     pub const UV_NO_EDITABLE: &'static str = "UV_NO_EDITABLE";
 
+    /// Equivalent to the `--dev` command-line argument. If set, uv will include
+    /// development dependencies.
+    pub const UV_DEV: &'static str = "UV_DEV";
+
+    /// Equivalent to the `--no-dev` command-line argument. If set, uv will exclude
+    /// development dependencies.
+    pub const UV_NO_DEV: &'static str = "UV_NO_DEV";
+
     /// Equivalent to the `--no-binary` command-line argument. If set, uv will install
     /// all packages from source. The resolver will still use pre-built wheels to
     /// extract package metadata, if available.
@@ -320,6 +335,26 @@ impl EnvVars {
     /// Distributions can be read from a local directory by using the `file://` URL scheme.
     pub const UV_PYPY_INSTALL_MIRROR: &'static str = "UV_PYPY_INSTALL_MIRROR";
 
+    /// Pin managed CPython versions to a specific build version.
+    ///
+    /// For CPython, this should be the build date (e.g., "20250814").
+    pub const UV_PYTHON_CPYTHON_BUILD: &'static str = "UV_PYTHON_CPYTHON_BUILD";
+
+    /// Pin managed PyPy versions to a specific build version.
+    ///
+    /// For PyPy, this should be the PyPy version (e.g., "7.3.20").
+    pub const UV_PYTHON_PYPY_BUILD: &'static str = "UV_PYTHON_PYPY_BUILD";
+
+    /// Pin managed GraalPy versions to a specific build version.
+    ///
+    /// For GraalPy, this should be the GraalPy version (e.g., "24.2.2").
+    pub const UV_PYTHON_GRAALPY_BUILD: &'static str = "UV_PYTHON_GRAALPY_BUILD";
+
+    /// Pin managed Pyodide versions to a specific build version.
+    ///
+    /// For Pyodide, this should be the Pyodide version (e.g., "0.28.1").
+    pub const UV_PYTHON_PYODIDE_BUILD: &'static str = "UV_PYTHON_PYODIDE_BUILD";
+
     /// Equivalent to the `--clear` command-line argument. If set, uv will remove any
     /// existing files or directories at the target path.
     pub const UV_VENV_CLEAR: &'static str = "UV_VENV_CLEAR";
@@ -496,6 +531,18 @@ impl EnvVars {
     /// Defaults to `13.0`, the least-recent non-EOL macOS version at time of writing.
     pub const MACOSX_DEPLOYMENT_TARGET: &'static str = "MACOSX_DEPLOYMENT_TARGET";
 
+    /// Used with `--python-platform arm64-apple-ios` and related variants to set the
+    /// deployment target (i.e., the minimum supported iOS version).
+    ///
+    /// Defaults to `13.0`.
+    pub const IPHONEOS_DEPLOYMENT_TARGET: &'static str = "IPHONEOS_DEPLOYMENT_TARGET";
+
+    /// Used with `--python-platform aarch64-linux-android` and related variants to set the
+    /// Android API level. (i.e., the minimum supported Android API level).
+    ///
+    /// Defaults to `24`.
+    pub const ANDROID_API_LEVEL: &'static str = "ANDROID_API_LEVEL";
+
     /// Disables colored output (takes precedence over `FORCE_COLOR`).
     ///
     /// See [no-color.org](https://no-color.org).
@@ -554,6 +601,10 @@ impl EnvVars {
     #[attr_hidden]
     pub const GIT_ALLOW_PROTOCOL: &'static str = "GIT_ALLOW_PROTOCOL";
 
+    /// Sets the SSH command used when Git tries to establish a connection using SSH.
+    #[attr_hidden]
+    pub const GIT_SSH_COMMAND: &'static str = "GIT_SSH_COMMAND";
+
     /// Disable interactive git prompts in terminals, e.g., for credentials. Does not disable
     /// GUI prompts.
     #[attr_hidden]
@@ -634,6 +685,18 @@ impl EnvVars {
     /// for more.
     pub const RUST_LOG: &'static str = "RUST_LOG";
 
+    /// If set, it can be used to display more stack trace details when a panic occurs.
+    /// This is used by uv particularly on windows to show more details during a platform exception.
+    ///
+    /// For example:
+    ///
+    /// * `RUST_BACKTRACE=1` will print a short backtrace.
+    /// * `RUST_BACKTRACE=full` will print a full backtrace.
+    ///
+    /// See the [Rust backtrace documentation](https://doc.rust-lang.org/std/backtrace/index.html)
+    /// for more.
+    pub const RUST_BACKTRACE: &'static str = "RUST_BACKTRACE";
+
     /// Add additional context and structure to log messages.
     ///
     /// If logging is not enabled, e.g., with `RUST_LOG` or `-v`, this has no effect.
@@ -694,6 +757,10 @@ impl EnvVars {
     #[attr_hidden]
     pub const ROOT_PATH: &'static str = "ROOT_PATH";
 
+    /// Used in testing extra build dependencies.
+    #[attr_hidden]
+    pub const EXPECTED_ANYIO_VERSION: &'static str = "EXPECTED_ANYIO_VERSION";
+
     /// Used to set test credentials for keyring tests.
     #[attr_hidden]
     pub const KEYRING_TEST_CREDENTIALS: &'static str = "KEYRING_TEST_CREDENTIALS";
@@ -709,6 +776,14 @@ impl EnvVars {
     #[attr_hidden]
     pub const UV_TEST_INDEX_URL: &'static str = "UV_TEST_INDEX_URL";
 
+    /// Used for testing named indexes in tests.
+    #[attr_hidden]
+    pub const UV_INDEX_MY_INDEX_USERNAME: &'static str = "UV_INDEX_MY_INDEX_USERNAME";
+
+    /// Used for testing named indexes in tests.
+    #[attr_hidden]
+    pub const UV_INDEX_MY_INDEX_PASSWORD: &'static str = "UV_INDEX_MY_INDEX_PASSWORD";
+
     /// Used to set the GitHub fast-path url for tests.
     #[attr_hidden]
     pub const UV_GITHUB_FAST_PATH_URL: &'static str = "UV_GITHUB_FAST_PATH_URL";
@@ -793,4 +868,27 @@ impl EnvVars {
 
     /// Disable Hugging Face authentication, even if `HF_TOKEN` is set.
     pub const UV_NO_HF_TOKEN: &'static str = "UV_NO_HF_TOKEN";
+
+    /// The URL of the pyx Simple API server.
+    pub const PYX_API_URL: &'static str = "PYX_API_URL";
+
+    /// The domain of the pyx CDN.
+    pub const PYX_CDN_DOMAIN: &'static str = "PYX_CDN_DOMAIN";
+
+    /// The pyx API key (e.g., `sk-pyx-...`).
+    pub const PYX_API_KEY: &'static str = "PYX_API_KEY";
+
+    /// The pyx API key, for backwards compatibility.
+    #[attr_hidden]
+    pub const UV_API_KEY: &'static str = "UV_API_KEY";
+
+    /// The pyx authentication token (e.g., `eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9...`), as output by `uv auth token`.
+    pub const PYX_AUTH_TOKEN: &'static str = "PYX_AUTH_TOKEN";
+
+    /// The pyx authentication token, for backwards compatibility.
+    #[attr_hidden]
+    pub const UV_AUTH_TOKEN: &'static str = "UV_AUTH_TOKEN";
+
+    /// Specifies the directory where uv stores pyx credentials.
+    pub const PYX_CREDENTIALS_DIR: &'static str = "PYX_CREDENTIALS_DIR";
 }
