@@ -8,14 +8,17 @@ use tracing::{debug, trace};
 use uv_cache::{Cache, Refresh};
 use uv_cache_info::Timestamp;
 use uv_client::BaseClientBuilder;
-use uv_configuration::{Concurrency, Constraints, DryRun, Preview, Reinstall, Upgrade};
+use uv_configuration::{Concurrency, Constraints, DryRun, Reinstall, TargetTriple, Upgrade};
+use uv_distribution::LoweredExtraBuildDependencies;
 use uv_distribution_types::{
-    NameRequirementSpecification, Requirement, RequirementSource,
+    ExtraBuildRequires, NameRequirementSpecification, Requirement, RequirementSource,
     UnresolvedRequirementSpecification,
 };
+use uv_installer::{SatisfiesResult, SitePackages};
 use uv_normalize::PackageName;
 use uv_pep440::{VersionSpecifier, VersionSpecifiers};
 use uv_pep508::MarkerTree;
+use uv_preview::Preview;
 use uv_python::{
     EnvironmentPreference, PythonDownloads, PythonInstallation, PythonPreference, PythonRequest,
 };
@@ -23,11 +26,12 @@ use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_settings::{PythonInstallMirrors, ResolverInstallerOptions, ToolOptions};
 use uv_tool::InstalledTools;
 use uv_warnings::warn_user;
-use uv_workspace::{WorkspaceCache, pyproject::ExtraBuildDependencies};
+use uv_workspace::WorkspaceCache;
 
 use crate::commands::ExitStatus;
 use crate::commands::pip::loggers::{DefaultInstallLogger, DefaultResolveLogger};
 use crate::commands::pip::operations::{self, Modifications};
+use crate::commands::pip::{resolution_markers, resolution_tags};
 use crate::commands::project::{
     EnvironmentSpecification, PlatformState, ProjectError, resolve_environment, resolve_names,
     sync_environment, update_environment,
@@ -38,7 +42,7 @@ use crate::commands::tool::common::{
 use crate::commands::tool::{Target, ToolRequest};
 use crate::commands::{diagnostics, reporters::PythonDownloadReporter};
 use crate::printer::Printer;
-use crate::settings::{NetworkSettings, ResolverInstallerSettings, ResolverSettings};
+use crate::settings::{ResolverInstallerSettings, ResolverSettings};
 
 /// Install a tool.
 #[allow(clippy::fn_params_excessive_bools)]
@@ -50,12 +54,14 @@ pub(crate) async fn install(
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
     build_constraints: &[RequirementsSource],
+    entrypoints: &[PackageName],
     python: Option<String>,
+    python_platform: Option<TargetTriple>,
     install_mirrors: PythonInstallMirrors,
     force: bool,
     options: ResolverInstallerOptions,
     settings: ResolverInstallerSettings,
-    network_settings: NetworkSettings,
+    client_builder: BaseClientBuilder<'_>,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
     installer_metadata: bool,
@@ -64,12 +70,6 @@ pub(crate) async fn install(
     printer: Printer,
     preview: Preview,
 ) -> Result<ExitStatus> {
-    let client_builder = BaseClientBuilder::new()
-        .retries_from_env()?
-        .connectivity(network_settings.connectivity)
-        .native_tls(network_settings.native_tls)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone());
-
     let reporter = PythonDownloadReporter::single(printer);
 
     let python_request = python.as_deref().map(PythonRequest::parse);
@@ -96,12 +96,6 @@ pub(crate) async fn install(
     let state = PlatformState::default();
     let workspace_cache = WorkspaceCache::default();
 
-    let client_builder = BaseClientBuilder::new()
-        .retries_from_env()?
-        .connectivity(network_settings.connectivity)
-        .native_tls(network_settings.native_tls)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone());
-
     // Parse the input requirement.
     let request = ToolRequest::parse(&package, from.as_deref())?;
 
@@ -113,7 +107,7 @@ pub(crate) async fn install(
     };
 
     // Resolve the `--from` requirement.
-    let from = match &request {
+    let requirement = match &request {
         // Ex) `ruff`
         ToolRequest::Package {
             executable,
@@ -146,7 +140,7 @@ pub(crate) async fn install(
                 requirement,
                 &interpreter,
                 &settings,
-                &network_settings,
+                &client_builder,
                 &state,
                 concurrency,
                 &cache,
@@ -219,22 +213,21 @@ pub(crate) async fn install(
         }
         // Ex) `python`
         ToolRequest::Python { .. } => {
-            return Err(anyhow::anyhow!(
+            bail!(
                 "Cannot install Python with `{}`. Did you mean to use `{}`?",
                 "uv tool install".cyan(),
                 "uv python install".cyan(),
-            ));
+            );
         }
     };
 
+    let package_name = &requirement.name;
+
     // If the user passed, e.g., `ruff@latest`, we need to mark it as upgradable.
     let settings = if request.is_latest() {
         ResolverInstallerSettings {
             resolver: ResolverSettings {
-                upgrade: settings
-                    .resolver
-                    .upgrade
-                    .combine(Upgrade::package(from.name.clone())),
+                upgrade: Upgrade::package(package_name.clone()).combine(settings.resolver.upgrade),
                 ..settings.resolver
             },
             ..settings
@@ -246,9 +239,7 @@ pub(crate) async fn install(
     // If the user passed `--force`, it implies `--reinstall-package <from>`
     let settings = if force {
         ResolverInstallerSettings {
-            reinstall: settings
-                .reinstall
-                .combine(Reinstall::package(from.name.clone())),
+            reinstall: Reinstall::package(package_name.clone()).combine(settings.reinstall),
             ..settings
         }
     } else {
@@ -268,13 +259,13 @@ pub(crate) async fn install(
     // Resolve the `--from` and `--with` requirements.
     let requirements = {
         let mut requirements = Vec::with_capacity(1 + with.len());
-        requirements.push(from.clone());
+        requirements.push(requirement.clone());
         requirements.extend(
             resolve_names(
                 spec.requirements.clone(),
                 &interpreter,
                 &settings,
-                &network_settings,
+                &client_builder,
                 &state,
                 concurrency,
                 &cache,
@@ -299,7 +290,7 @@ pub(crate) async fn install(
         spec.overrides,
         &interpreter,
         &settings,
-        &network_settings,
+        &client_builder,
         &state,
         concurrency,
         &cache,
@@ -332,16 +323,16 @@ pub(crate) async fn install(
     // (If we find existing entrypoints later on, and the tool _doesn't_ exist, we'll avoid removing
     // the external tool's entrypoints (without `--force`).)
     let (existing_tool_receipt, invalid_tool_receipt) =
-        match installed_tools.get_tool_receipt(&from.name) {
+        match installed_tools.get_tool_receipt(package_name) {
             Ok(None) => (None, false),
             Ok(Some(receipt)) => (Some(receipt), false),
             Err(_) => {
                 // If the tool is not installed properly, remove the environment and continue.
-                match installed_tools.remove_environment(&from.name) {
+                match installed_tools.remove_environment(package_name) {
                     Ok(()) => {
                         warn_user!(
-                            "Removed existing `{from}` with invalid receipt",
-                            from = from.name.cyan()
+                            "Removed existing `{}` with invalid receipt",
+                            package_name.cyan()
                         );
                     }
                     Err(uv_tool::Error::Io(err)) if err.kind() == std::io::ErrorKind::NotFound => {}
@@ -355,56 +346,90 @@ pub(crate) async fn install(
 
     let existing_environment =
         installed_tools
-            .get_environment(&from.name, &cache)?
+            .get_environment(package_name, &cache)?
             .filter(|environment| {
                 if environment.uses(&interpreter) {
                     trace!(
                         "Existing interpreter matches the requested interpreter for `{}`: {}",
-                        from.name,
+                        package_name,
                         environment.interpreter().sys_executable().display()
                     );
                     true
                 } else {
                     let _ = writeln!(
                         printer.stderr(),
-                        "Ignoring existing environment for `{from}`: the requested Python interpreter does not match the environment interpreter",
-                        from = from.name.cyan(),
+                        "Ignoring existing environment for `{}`: the requested Python interpreter does not match the environment interpreter",
+                        package_name.cyan(),
                     );
                     false
                 }
             });
 
     // If the requested and receipt requirements are the same...
-    if existing_environment
-        .as_ref()
-        .filter(|_| {
-            // And the user didn't request a reinstall or upgrade...
-            !request.is_latest()
-                && settings.reinstall.is_none()
-                && settings.resolver.upgrade.is_none()
-        })
-        .is_some()
-    {
+    if let Some(environment) = existing_environment.as_ref().filter(|_| {
+        // And the user didn't request a reinstall or upgrade...
+        !request.is_latest() && settings.reinstall.is_none() && settings.resolver.upgrade.is_none()
+    }) {
         if let Some(tool_receipt) = existing_tool_receipt.as_ref() {
             if requirements == tool_receipt.requirements()
                 && constraints == tool_receipt.constraints()
                 && overrides == tool_receipt.overrides()
                 && build_constraints == tool_receipt.build_constraints()
             {
-                if *tool_receipt.options() != options {
-                    // ...but the options differ, we need to update the receipt.
-                    installed_tools
-                        .add_tool_receipt(&from.name, tool_receipt.clone().with_options(options))?;
+                let ResolverInstallerSettings {
+                    resolver:
+                        ResolverSettings {
+                            config_setting,
+                            config_settings_package,
+                            extra_build_dependencies,
+                            extra_build_variables,
+                            ..
+                        },
+                    ..
+                } = &settings;
+
+                // Lower the extra build dependencies, if any.
+                let extra_build_requires = LoweredExtraBuildDependencies::from_non_lowered(
+                    extra_build_dependencies.clone(),
+                )
+                .into_inner();
+
+                // Determine the markers and tags to use for the resolution.
+                let markers = resolution_markers(None, python_platform.as_ref(), &interpreter);
+                let tags = resolution_tags(None, python_platform.as_ref(), &interpreter)?;
+
+                // Check if the installed packages meet the requirements.
+                let site_packages = SitePackages::from_environment(environment)?;
+                if matches!(
+                    site_packages.satisfies_requirements(
+                        requirements.iter(),
+                        constraints.iter(),
+                        overrides.iter(),
+                        &markers,
+                        &tags,
+                        config_setting,
+                        config_settings_package,
+                        &extra_build_requires,
+                        extra_build_variables,
+                    ),
+                    Ok(SatisfiesResult::Fresh { .. })
+                ) {
+                    // Then we're done! Though we might need to update the receipt.
+                    if *tool_receipt.options() != options {
+                        installed_tools.add_tool_receipt(
+                            package_name,
+                            tool_receipt.clone().with_options(options),
+                        )?;
+                    }
+
+                    writeln!(
+                        printer.stderr(),
+                        "`{}` is already installed",
+                        requirement.cyan()
+                    )?;
+
+                    return Ok(ExitStatus::Success);
                 }
-
-                // We're done, though we might need to update the receipt.
-                writeln!(
-                    printer.stderr(),
-                    "`{from}` is already installed",
-                    from = from.cyan()
-                )?;
-
-                return Ok(ExitStatus::Success);
             }
         }
     }
@@ -438,10 +463,11 @@ pub(crate) async fn install(
             environment,
             spec,
             Modifications::Exact,
+            python_platform.as_ref(),
             Constraints::from_requirements(build_constraints.iter().cloned()),
-            uv_distribution::ExtraBuildRequires::from_lowered(ExtraBuildDependencies::default()),
+            ExtraBuildRequires::default(),
             &settings,
-            &network_settings,
+            &client_builder,
             &state,
             Box::new(DefaultResolveLogger),
             Box::new(DefaultInstallLogger),
@@ -457,9 +483,11 @@ pub(crate) async fn install(
         {
             Ok(update) => update.into_environment(),
             Err(ProjectError::Operation(err)) => {
-                return diagnostics::OperationDiagnostic::native_tls(network_settings.native_tls)
-                    .report(err)
-                    .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
+                return diagnostics::OperationDiagnostic::native_tls(
+                    client_builder.is_native_tls(),
+                )
+                .report(err)
+                .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
             }
             Err(err) => return Err(err.into()),
         };
@@ -479,9 +507,10 @@ pub(crate) async fn install(
         let resolution = resolve_environment(
             spec.clone(),
             &interpreter,
+            python_platform.as_ref(),
             Constraints::from_requirements(build_constraints.iter().cloned()),
             &settings.resolver,
-            &network_settings,
+            &client_builder,
             &state,
             Box::new(DefaultResolveLogger),
             concurrency,
@@ -518,7 +547,7 @@ pub(crate) async fn install(
                     .ok()
                     .flatten() else {
                         return diagnostics::OperationDiagnostic::native_tls(
-                            network_settings.native_tls,
+                            client_builder.is_native_tls(),
                         )
                         .report(err)
                         .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
@@ -533,9 +562,10 @@ pub(crate) async fn install(
                     match resolve_environment(
                         spec,
                         &interpreter,
+                        python_platform.as_ref(),
                         Constraints::from_requirements(build_constraints.iter().cloned()),
                         &settings.resolver,
-                        &network_settings,
+                        &client_builder,
                         &state,
                         Box::new(DefaultResolveLogger),
                         concurrency,
@@ -548,7 +578,7 @@ pub(crate) async fn install(
                         Ok(resolution) => (resolution, interpreter),
                         Err(ProjectError::Operation(err)) => {
                             return diagnostics::OperationDiagnostic::native_tls(
-                                network_settings.native_tls,
+                                client_builder.is_native_tls(),
                             )
                             .report(err)
                             .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
@@ -560,7 +590,7 @@ pub(crate) async fn install(
             },
         };
 
-        let environment = installed_tools.create_environment(&from.name, interpreter, preview)?;
+        let environment = installed_tools.create_environment(package_name, interpreter, preview)?;
 
         // At this point, we removed any existing environment, so we should remove any of its
         // executables.
@@ -575,7 +605,7 @@ pub(crate) async fn install(
             Modifications::Exact,
             Constraints::from_requirements(build_constraints.iter().cloned()),
             (&settings).into(),
-            &network_settings,
+            &client_builder,
             &state,
             Box::new(DefaultInstallLogger),
             installer_metadata,
@@ -587,14 +617,16 @@ pub(crate) async fn install(
         .await
         .inspect_err(|_| {
             // If we failed to sync, remove the newly created environment.
-            debug!("Failed to sync environment; removing `{}`", from.name);
-            let _ = installed_tools.remove_environment(&from.name);
+            debug!("Failed to sync environment; removing `{}`", package_name);
+            let _ = installed_tools.remove_environment(package_name);
         }) {
             Ok(environment) => environment,
             Err(ProjectError::Operation(err)) => {
-                return diagnostics::OperationDiagnostic::native_tls(network_settings.native_tls)
-                    .report(err)
-                    .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
+                return diagnostics::OperationDiagnostic::native_tls(
+                    client_builder.is_native_tls(),
+                )
+                .report(err)
+                .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
             }
             Err(err) => return Err(err.into()),
         }
@@ -602,9 +634,10 @@ pub(crate) async fn install(
 
     finalize_tool_install(
         &environment,
-        &from.name,
+        package_name,
+        entrypoints,
         &installed_tools,
-        options,
+        &options,
         force || invalid_tool_receipt,
         python_request,
         requirements,
@@ -612,5 +645,7 @@ pub(crate) async fn install(
         overrides,
         build_constraints,
         printer,
-    )
+    )?;
+
+    Ok(ExitStatus::Success)
 }
