@@ -18,11 +18,11 @@ use indoc::formatdoc;
 use itertools::Itertools;
 use predicates::prelude::predicate;
 use regex::Regex;
-
 use tokio::io::AsyncWriteExt;
+
 use uv_cache::{Cache, CacheBucket};
-use uv_configuration::Preview;
 use uv_fs::Simplified;
+use uv_preview::Preview;
 use uv_python::managed::ManagedPythonInstallations;
 use uv_python::{
     EnvironmentPreference, PythonInstallation, PythonPreference, PythonRequest, PythonVersion,
@@ -220,28 +220,32 @@ impl TestContext {
     /// and `.exe` suffixes.
     #[must_use]
     pub fn with_filtered_python_names(mut self) -> Self {
-        use env::consts::EXE_SUFFIX;
-        let exe_suffix = regex::escape(EXE_SUFFIX);
+        for name in ["python", "pypy"] {
+            // Note we strip version numbers from the executable names because, e.g., on Windows
+            // `python.exe` is the equivalent to a Unix `python3.12`.`
+            let suffix = if cfg!(windows) {
+                // On Windows, we'll require a `.exe` suffix for disambiguation
+                // We'll also strip version numbers if present, which is not common for `python.exe`
+                // but can occur for, e.g., `pypy3.12.exe`
+                let exe_suffix = regex::escape(env::consts::EXE_SUFFIX);
+                format!(r"(\d\.\d+|\d)?{exe_suffix}")
+            } else {
+                // On Unix, we'll strip version numbers
+                if name == "python" {
+                    // We can't require them in this case since `/python` is common
+                    r"(\d\.\d+|\d)?".to_string()
+                } else {
+                    // However, for other names we'll require them to avoid over-matching
+                    r"(\d\.\d+|\d)".to_string()
+                }
+            };
 
-        self.filters.push((
-            format!(r"python\d.\d\d{exe_suffix}"),
-            "[PYTHON]".to_string(),
-        ));
-        self.filters
-            .push((format!(r"python\d{exe_suffix}"), "[PYTHON]".to_string()));
-
-        if cfg!(windows) {
-            // On Windows, we want to filter out all `python.exe` instances
-            self.filters
-                .push((format!(r"python{exe_suffix}"), "[PYTHON]".to_string()));
-            // Including ones where we'd already stripped the `.exe` in another filter
-            self.filters
-                .push((r"[\\/]python".to_string(), "/[PYTHON]".to_string()));
-        } else {
-            // On Unix, it's a little trickier — we don't want to clobber use of `python` in the
-            // middle of something else, e.g., `cpython`. For this reason, we require a leading `/`.
-            self.filters
-                .push((format!(r"/python{exe_suffix}"), "/[PYTHON]".to_string()));
+            self.filters.push((
+                // We use a leading path separator to help disambiguate cases where the name is not
+                // used in a path.
+                format!(r"[\\/]{name}{suffix}"),
+                format!("/[{}]", name.to_uppercase()),
+            ));
         }
 
         self
@@ -270,15 +274,34 @@ impl TestContext {
     /// the virtual environment equivalent.
     #[must_use]
     pub fn with_filtered_python_install_bin(mut self) -> Self {
+        // We don't want to eagerly match paths that aren't actually Python executables, so we
+        // do our best to detect that case
+        let suffix = if cfg!(windows) {
+            let exe_suffix = regex::escape(env::consts::EXE_SUFFIX);
+            // On Windows, we usually don't have a version attached but we might, e.g., for pypy3.12
+            format!(r"(\d\.\d+|\d)?{exe_suffix}")
+        } else {
+            // On Unix, we'll require a version to be attached to avoid over-matching
+            r"\d\.\d+|\d".to_string()
+        };
+
         if cfg!(unix) {
             self.filters.push((
-                r"[\\/]bin/python".to_string(),
-                "/[INSTALL-BIN]/python".to_string(),
+                format!(r"[\\/]bin/python({suffix})"),
+                "/[INSTALL-BIN]/python$1".to_string(),
+            ));
+            self.filters.push((
+                format!(r"[\\/]bin/pypy({suffix})"),
+                "/[INSTALL-BIN]/pypy$1".to_string(),
             ));
         } else {
             self.filters.push((
-                r"[\\/]python".to_string(),
-                "/[INSTALL-BIN]/python".to_string(),
+                format!(r"[\\/]python({suffix})"),
+                "/[INSTALL-BIN]/python$1".to_string(),
+            ));
+            self.filters.push((
+                format!(r"[\\/]pypy({suffix})"),
+                "/[INSTALL-BIN]/pypy$1".to_string(),
             ));
         }
         self
@@ -592,6 +615,36 @@ impl TestContext {
             filters.push((r"exit code: ".to_string(), "exit status: ".to_string()));
         }
 
+        for (version, executable) in &python_versions {
+            // Add filtering for the interpreter path
+            filters.extend(
+                Self::path_patterns(executable)
+                    .into_iter()
+                    .map(|pattern| (pattern.to_string(), format!("[PYTHON-{version}]"))),
+            );
+
+            // And for the symlink we created in the test the Python path
+            filters.extend(
+                Self::path_patterns(python_dir.join(version.to_string()))
+                    .into_iter()
+                    .map(|pattern| {
+                        (
+                            format!("{pattern}[a-zA-Z0-9]*"),
+                            format!("[PYTHON-{version}]"),
+                        )
+                    }),
+            );
+
+            // Add Python patch version filtering unless explicitly requested to ensure
+            // snapshots are patch version agnostic when it is not a part of the test.
+            if version.patch().is_none() {
+                filters.push((
+                    format!(r"({})\.\d+", regex::escape(version.to_string().as_str())),
+                    "$1.[X]".to_string(),
+                ));
+            }
+        }
+
         filters.extend(
             Self::path_patterns(&bin_dir)
                 .into_iter()
@@ -634,35 +687,6 @@ impl TestContext {
         ));
         filters.push((r"[\\/]Lib[\\/]".to_string(), "/[PYTHON-LIB]/".to_string()));
 
-        for (version, executable) in &python_versions {
-            // Add filtering for the interpreter path
-            filters.extend(
-                Self::path_patterns(executable)
-                    .into_iter()
-                    .map(|pattern| (pattern.to_string(), format!("[PYTHON-{version}]"))),
-            );
-
-            // And for the symlink we created in the test the Python path
-            filters.extend(
-                Self::path_patterns(python_dir.join(version.to_string()))
-                    .into_iter()
-                    .map(|pattern| {
-                        (
-                            format!("{pattern}[a-zA-Z0-9]*"),
-                            format!("[PYTHON-{version}]"),
-                        )
-                    }),
-            );
-
-            // Add Python patch version filtering unless explicitly requested to ensure
-            // snapshots are patch version agnostic when it is not a part of the test.
-            if version.patch().is_none() {
-                filters.push((
-                    format!(r"({})\.\d+", regex::escape(version.to_string().as_str())),
-                    "$1.[X]".to_string(),
-                ));
-            }
-        }
         filters.extend(
             Self::path_patterns(&temp_dir)
                 .into_iter()
@@ -830,6 +854,10 @@ impl TestContext {
             .env(EnvVars::HOME, self.home_dir.as_os_str())
             .env(EnvVars::APPDATA, self.home_dir.as_os_str())
             .env(EnvVars::USERPROFILE, self.home_dir.as_os_str())
+            .env(
+                EnvVars::XDG_DATA_HOME,
+                self.home_dir.join("data").as_os_str(),
+            )
             .env(EnvVars::UV_PYTHON_INSTALL_DIR, "")
             // Installations are not allowed by default; see `Self::with_managed_python_dirs`
             .env(EnvVars::UV_PYTHON_DOWNLOADS, "never")
@@ -844,7 +872,6 @@ impl TestContext {
             .env_remove(EnvVars::UV_CACHE_DIR)
             .env_remove(EnvVars::UV_TOOL_BIN_DIR)
             .env_remove(EnvVars::XDG_CONFIG_HOME)
-            .env_remove(EnvVars::XDG_DATA_HOME)
             // I believe the intent of all tests is that they are run outside the
             // context of an existing git repository. And when they aren't, state
             // from the parent git repository can bleed into the behavior of `uv
@@ -995,6 +1022,14 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv format` command with options shared across scenarios.
+    pub fn format(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("format");
+        self.add_shared_options(&mut command, false);
+        command
+    }
+
     /// Create a `uv build` command with options shared across scenarios.
     pub fn build(&self) -> Command {
         let mut command = Self::new_command();
@@ -1242,6 +1277,42 @@ impl TestContext {
         command
     }
 
+    /// Create a `uv auth login` command.
+    pub fn auth_login(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("auth").arg("login");
+        self.add_shared_options(&mut command, false);
+        command
+    }
+
+    /// Create a `uv auth logout` command.
+    pub fn auth_logout(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("auth").arg("logout");
+        self.add_shared_options(&mut command, false);
+        command
+    }
+
+    /// Create a `uv auth token` command.
+    pub fn auth_token(&self) -> Command {
+        let mut command = Self::new_command();
+        command.arg("auth").arg("token");
+        self.add_shared_options(&mut command, false);
+        command
+    }
+
+    /// Set `HOME` to the real home directory.
+    ///
+    /// We need this for testing commands which use the macOS keychain.
+    #[must_use]
+    pub fn with_real_home(mut self) -> Self {
+        if let Some(home) = env::var_os(EnvVars::HOME) {
+            self.extra_env
+                .push((EnvVars::HOME.to_string().into(), home));
+        }
+        self
+    }
+
     /// Run the given python code and check whether it succeeds.
     pub fn assert_command(&self, command: &str) -> Assert {
         self.python_command()
@@ -1787,7 +1858,7 @@ pub async fn download_to_disk(url: &str, path: &Path) {
         .map(|h| uv_configuration::TrustedHost::from_str(h).unwrap())
         .collect();
 
-    let client = uv_client::BaseClientBuilder::new()
+    let client = uv_client::BaseClientBuilder::default()
         .allow_insecure_host(trusted_hosts)
         .build();
     let url = url.parse().unwrap();
@@ -1798,7 +1869,7 @@ pub async fn download_to_disk(url: &str, path: &Path) {
         .await
         .unwrap();
 
-    let mut file = tokio::fs::File::create(path).await.unwrap();
+    let mut file = fs_err::tokio::File::create(path).await.unwrap();
     let mut stream = response.bytes_stream();
     while let Some(chunk) = stream.next().await {
         file.write_all(&chunk.unwrap()).await.unwrap();
